# 0.1.0-SNAPSHOT

Third snapshot release of RainbowCake.

## Update steps:

Update your dependency version:

```groovy
implementation 'co.zsmb:rainbow-cake:0.1.0-SNAPSHOT'
```

Optionally, update your screen templates, which can now be done in a more convenient way via [a simple `git clone`](https://gitlab.autsoft.hu/AutSoft/AndroidChapter/rainbow-cake/rainbow-cake-templates#git-cloned-installation).

## What's new:

### Shared ViewModels between Fragments

ViewModels by default are scoped to their Fragment, meaning a new instance is created for every new instance of the Fragment (barring configuration changes), and they are cleared when their Fragment is destroyed (as in their lifecycle completely ends).

There are use cases where it would make sense to share ViewModel instances between Fragments, and the `getViewModelFromFactory` method now provides an opportunity for this in the form of an optional parameter. ViewModels may now be scoped to the current Activity or to a parent Fragment. For details, see the documentation [here](https://gitlab.autsoft.hu/AutSoft/AndroidChapter/rainbow-cake/rainbow-cake/blob/b6d914c3f5949935b5bf5dada5a555f80960af9b/rainbow-cake/src/main/java/hu/autsoft/rainbowcake/base/ViewModelScope.kt).

A demo showcasing a `ViewPager` where the pages share a ViewModel scoped to their parent Fragment is also available [here](https://gitlab.autsoft.hu/AutSoft/AndroidChapter/rainbow-cake/rainbow-cake/tree/dev/demo/src/main/java/hu/autsoft/rainbowcake/demo/ui/sharedvmpager).

### Navigation improvements

The `Navigator` provided by the library contains a fade animation between screen changes by default. It's not possible to override this default behaviour.

You can override it globally, by providing new values for certain properties in your Activity that inherits from `NavActivity`:

```kotlin
class MainActivity : SimpleNavActivity() {

    override val defaultEnterAnim: Int = R.anim.slide_in_right
    override val defaultExitAnim: Int = R.anim.slide_out_left
    override val defaultPopEnterAnim: Int = R.anim.slide_in_left
    override val defaultPopExitAnim: Int = R.anim.slide_out_right
    
}
```

You can also override animations one by one, by using overloads of the `add` and `replace` methods:

```kotlin
navigator?.add(SomeFragment(),
    enterAnim = R.anim.slide_in_right,
    exitAnim = R.anim.slide_out_left,
    popEnterAnim = R.anim.slide_in_left,
    popExitAnim = R.anim.slide_out_right
)
```

Note that a simple `0` may be used for any of these values to disable an animation altogether.

Be sure to check the documentation for all of the properties and methods mentioned above, as they contain much more information.

### New argument handling methods

New methods have been added to support Fragment arguments with `Boolean` and `Parcelable` types. These conform to the naming convention of existing argument handling methods, see them [here](https://gitlab.autsoft.hu/AutSoft/AndroidChapter/rainbow-cake/rainbow-cake/commit/4352e06abf9b8c456100b7c59ffabf6a9d881717) and [here](https://gitlab.autsoft.hu/AutSoft/AndroidChapter/rainbow-cake/rainbow-cake/commit/ba0ebb6cae17bc99dd02b8ee2ef19bfbd0e2bd02).

### New `executeCancellable` method

Previously, any coroutines started by making `execute` calls in the `ViewModel` were only cancelled when the `ViewModel` was cleared. If you need to manage the `Job` representing coroutines manually instead, you can now do so with the `executeCancellable` method:

```kotlin
class MyViewModel : JobViewModel<MyViewState>(Default) {

    private var loadingJob: Job? = null

    fun loadData() {
        loadingJob?.cancel()
        
        loadingJob = executeCancellable {
            // do something
        }
    }

}
```

Note that you shouldn't ever return this `Job` to your Fragment, so ***do not do this***, as the `loadData` method here has an implicit return type of `Job`, instead of `Unit`:

```kotlin
class MyViewModel : JobViewModel<MyViewState>(Default) {

    fun loadData() = executeCancellable {
        // do something
    }

}
```

### Bug fixes and performance improvements (yes, really)

- Fragment backstack management fixes around the `replace` operation of the `Navigator` implementation.
- The `rootJob` in `JobViewModel` is now a `SupervisorJob` so that it's not cancelled altogether if a child coroutine fails (based on [this](https://proandroiddev.com/kotlin-coroutines-patterns-anti-patterns-f9d12984c68e) article's advice).
- The `coroutineContext` used by the `CoroutineScope` in `JobViewModel` is now only created once at instantiation.
- Channel observations are now explicitly cleared when a `ChannelViewModel` is cleared.

#### Version updates: 

- Kotlin 1.3.11
- Coroutines 1.1.0
- Android Gradle Plugin 3.3.0
